/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: sc_printopt.cxx,v $
 *
 *  $Revision: 1.4 $
 *
 *  last change: $Author: rt $ $Date: 2006/07/25 08:06:37 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#ifdef PCH
#include "ui_pch.hxx"
#endif

#pragma hdrstop

#ifndef PCH
#include <sv.hxx>
#include <sbx.hxx>
#include <sfxitems.hxx>
#endif

#include "bf_sc.hrc" // -> SCSTR_UNDEFINED

#include "printopt.hxx"
#include "printopt.hrc"
namespace binfilter {

//========================================================================
ScPrintOptionsPage::ScPrintOptionsPage( Window* 			pParent,
										const SfxItemSet&	rCoreAttrs	) :
		SfxTabPage		( pParent,
						  ResId( RID_SCPAGE_PRINTOPTIONS ),
						  rCoreAttrs ),
		//
		aEdRepeatRow	( this, ResId( ED_REPEAT_ROW ) ),
		aEdRepeatCol	( this, ResId( ED_REPEAT_COL ) ),
		aFtRepeatRow	( this, ResId( FT_REPEAT_ROW ) ),
		aFtRepeatCol	( this, ResId( FT_REPEAT_COL ) ),
		//
		aEdScale		( this, ResId( ED_SCALE ) ),
		aFtScale		( this, ResId( FT_SCALE ) ),
		//
		aBtnHeader		( this, ResId( BTN_HEADER ) ),
		aBtnGrid		( this, ResId( BTN_GRID ) ),
		aBtnObjects 	( this, ResId( BTN_OBJECTS ) ),
		aBtnDiagrams	( this, ResId( BTN_DIAGRAMS ) ),
		aGbPrint		( this, ResId( GB_PRINT ) ),
		//
		aBtnTopDown 	( this, ResId( BTN_TOPDOWN ) ),
		aBtnLeftRight	( this, ResId( BTN_LEFTRIGHT ) ),
		aGbNumbering	( this, ResId( GB_NUMBERING ) ),
		//
		aBtnTable		( this, ResId( BTN_PAGES_TABLE ) ),
		aBtnFolder		( this, ResId( BTN_PAGES_FOLDER ) ),
		aBtnArea		( this, ResId( BTN_PAGES_AREA ) ),
		aLbArea 		( this, ResId( LB_PAGES_AREA ) ),
		aGbPages		( this, ResId( GB_PAGES ) ),
		//
		aBtnNotesNone	( this, ResId( BTN_NOTES_NONE ) ),
		aBtnNotesDocEnd ( this, ResId( BTN_NOTES_DOCEND ) ),
		aBtnNotesOnly	( this, ResId( BTN_NOTES_ONLY ) ),
		aBtnNotesPageEnd( this, ResId( BTN_NOTES_PAGEEND ) ),
		aGbNotes		( this, ResId( GB_NOTES ) )
{
	Init();
	FreeResource();
}

//------------------------------------------------------------------------

__EXPORT ScPrintOptionsPage::~ScPrintOptionsPage()
{
}

//------------------------------------------------------------------------

void ScPrintOptionsPage::Init()
{
	aBtnTable. SetClickHdl( LINK( this, ScPrintOptionsPage, AreaClickHdl) );
	aBtnFolder.SetClickHdl( LINK( this, ScPrintOptionsPage, AreaClickHdl) );
	aBtnArea.  SetClickHdl( LINK( this, ScPrintOptionsPage, AreaClickHdl) );

	// Dummy:
	aBtnTable.Check();
	aLbArea.Disable();
	aLbArea.Clear();
	aLbArea.InsertEntry( String( ResId( SCSTR_UNDEFINED ) ) );
	aLbArea.SelectEntryPos( 0 );
	aBtnDiagrams.Check();
	aBtnNotesNone.Check();
	aBtnLeftRight.Check();
	aEdScale.SetValue( 100 );
}

//------------------------------------------------------------------------

SfxTabPage* __EXPORT ScPrintOptionsPage::Create( Window*			 pParent,
										const SfxItemSet&	rAttrSet )
{
	return ( new ScPrintOptionsPage( pParent, rAttrSet ) );
}

//------------------------------------------------------------------------

BOOL ScPrintOptionsPage::FillItemSet( SfxItemSet& rCoreAttrs )
{
	return ( TRUE );
}

//------------------------------------------------------------------------

void ScPrintOptionsPage::Reset( const SfxItemSet& )
{
}

//------------------------------------------------------------------------
// Handler

long __EXPORT ScPrintOptionsPage::AreaClickHdl( RadioButton* pBtn )
{
	if (   (pBtn == &aBtnTable)
		|| (pBtn == &aBtnFolder)
		|| (pBtn == &aBtnArea) )
	{
		if ( aBtnArea.IsChecked() )
			aLbArea.Enable();
		else
			aLbArea.Disable();
	}

	return 0;
}


}
